<?php
namespace SIU\SQ\Lib;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_negocio;

abstract class transaccion
{
	protected $entorno;
	protected $datos;
	protected $id;
	
	function __construct($entorno, $id = null)
	{
		$this->entorno = $entorno;
		$this->id = $id;
	}

	/**
	 * @return entorno
	 */
	function entorno()
	{
		if(!isset($this->entorno)){
			throw new error("PROCESO - El entorno no esta cargado");
		}
		return $this->entorno;		
	}
	
	// Metodos obligatorios
	abstract public function procesar_especifico();
	
	public function procesar()
	{
		if (!isset($this->datos)) throw new error(__METHOD__ . ': no hay datos para procesar');
		try {
			$this->entorno()->db()->abrir_transaccion();
			
			$this->procesar_especifico();
			
			$this->entorno()->db()->cerrar_transaccion();
		} catch (error_db $e) {
			$this->entorno()->db()->abortar_transaccion($e->get_mensaje());
			throw $e;
		} catch (error $e) {
			$this->entorno()->db()->abortar_transaccion($e->get_mensaje());
			throw $e;
		}
	}

	function get_datos()
	{
		return $this->datos;
	}

	public function set_datos($datos)
	{
		$this->datos = $datos;
	}
	
	function get_id()
	{
		return $this->id;
	}
	
	function set_id($id)
	{
		$this->id = $id;
	}
	
	protected function validar_campos_obligatorios($datos, $campos_obligatorios, $prefijo_desc = '')
	{
		$error_obligatorios = false;
		foreach ($campos_obligatorios as $nombre_campo) {
			if (!in_array($nombre_campo, $datos, true)
				AND ( !isset($datos[$nombre_campo])
					OR $datos[$nombre_campo] === NULL)) {
				$error_obligatorios[] = $nombre_campo;
			}
		}
		if ($error_obligatorios) {
			throw new error_negocio($prefijo_desc . ": Los siguientes campos son obligatorios: '"
				. implode("', '", $error_obligatorios) . "'.");
		}
	}
}
